@@ -31,7 +31,6 @@ gi.require_versions({"Grl": "0.3"})
 from gi.repository import Gio, Grl, GLib, GObject
 
 from gnomemusic.coresong import CoreSong
-from gnomemusic.trackerwrapper import TrackerWrapper
 import gnomemusic.utils as utils
 
 
@@ -55,9 +54,6 @@ class GrlTrackerPlaylists(GObject.GObject):
         Grl.METADATA_KEY_URL
     ]
 
-    def __repr__(self):
-        return "<GrlTrackerPlaylists>"
-
     def __init__(
             self, source, coremodel, application, grilo, tracker_wrapper):
         """Initialize GrlTrackerPlaylists.
@@ -76,12 +72,14 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._coremodel = coremodel
         self._coreselection = application.props.coreselection
         self._grilo = grilo
+        self._log = application.props.log
         self._source = source
         self._model = self._coremodel.props.playlists
         self._model_filter = self._coremodel.props.playlists_filter
         self._user_model_filter = self._coremodel.props.user_playlists_filter
         self._pls_todelete = []
         self._tracker = tracker_wrapper.props.tracker
+        self._tracker_wrapper = tracker_wrapper
         self._window = application.props.window
 
         self._user_model_filter.set_filter_func(self._user_playlists_filter)
@@ -97,7 +95,7 @@ class GrlTrackerPlaylists(GObject.GObject):
             "source": self._source,
             "application": self._application,
             "grilo": self._grilo,
-            "tracker": self._tracker
+            "tracker_wrapper": self._tracker_wrapper
         }
 
         smart_playlists = {
@@ -141,7 +139,7 @@ class GrlTrackerPlaylists(GObject.GObject):
     def _add_user_playlist(
             self, source, op_id, media, remaining, data=None, error=None):
         if error:
-            print("ERROR", error)
+            self._log.warning("Error: {}".format(error))
             self._window.notifications_popup.pop_loading()
             return
         if not media:
@@ -151,7 +149,7 @@ class GrlTrackerPlaylists(GObject.GObject):
         playlist = Playlist(
             media=media, source=self._source, coremodel=self._coremodel,
             application=self._application, grilo=self._grilo,
-            tracker=self._tracker)
+            tracker_wrapper=self._tracker_wrapper)
 
         self._model.append(playlist)
         callback = data
@@ -187,12 +185,16 @@ class GrlTrackerPlaylists(GObject.GObject):
             return
 
         def _delete_cb(conn, res, data):
-            # FIXME: Check for failure.
-            conn.update_finish(res)
-            for idx, playlist_model in enumerate(self._model):
-                if playlist_model is playlist:
-                    self._model.remove(idx)
-                    break
+            try:
+                conn.update_finish(res)
+            except GLib.Error as error:
+                self._log.warning("Unable to delete playlist {}: {}".format(
+                    playlist.props.title, error.message))
+            else:
+                for idx, playlist_model in enumerate(self._model):
+                    if playlist_model is playlist:
+                        self._model.remove(idx)
+                        break
 
             self._model_filter.set_filter_func(self._playlists_filter)
             self._window.notifications_popup.pop_loading()
@@ -255,20 +257,6 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._tracker.update_blank_async(
             query, GLib.PRIORITY_LOW, None, _create_cb, None)
 
-    def check_smart_playlist_change(self):
-        """Check if smart playlists need to be updated.
-
-        A smart playlist needs to be updated in two cases:
-        * it is being played (active_playlist)
-        * it is visible in PlaylistsView
-        """
-        active_playlist = self._coremodel.props.active_playlist
-        if (active_playlist is not None
-                and active_playlist.props.is_smart is True):
-            active_playlist.update()
-        else:
-            self._coremodel.emit("smart-playlist-change")
-
 
 class Playlist(GObject.GObject):
     """ Base class of all playlists """
@@ -300,14 +288,11 @@ class Playlist(GObject.GObject):
     pl_id = GObject.Property(type=str, default=None)
     query = GObject.Property(type=str, default=None)
     tag_text = GObject.Property(type=str, default=None)
-    title = GObject.Property(type=str, default=None)
-
-    def __repr__(self):
-        return "<Playlist>"
 
     def __init__(
             self, media=None, query=None, tag_text=None, source=None,
-            coremodel=None, application=None, grilo=None, tracker=None):
+            coremodel=None, application=None, grilo=None,
+            tracker_wrapper=None):
 
         super().__init__()
         """Initialize a playlist
@@ -320,13 +305,16 @@ class Playlist(GObject.GObject):
        :param CoreModel coremodel: The CoreModel instance
        :param Application application: The Application instance
        :param CoreGrilo grilo: The CoreGrilo instance
-       :param TrackerWrapper tracker: The TrackerWrapper instance
+       :param TrackerWrapper tracker_wrapper: The TrackerWrapper
+            instance
         """
         if media:
             self.props.pl_id = media.get_id()
-            self.props.title = utils.get_media_title(media)
+            self._title = utils.get_media_title(media)
             self.props.count = media.get_childcount()
             self.props.creation_date = media.get_creation_date()
+        else:
+            self._title = None
 
         self.props.query = query
         self.props.tag_text = tag_text
@@ -335,7 +323,9 @@ class Playlist(GObject.GObject):
         self._coremodel = coremodel
         self._coreselection = application.props.coreselection
         self._grilo = grilo
-        self._tracker = tracker
+        self._log = application.props.log
+        self._tracker = tracker_wrapper.props.tracker
+        self._tracker_wrapper = tracker_wrapper
         self._window = application.props.window
 
         self._fast_options = Grl.OperationOptions()
@@ -418,7 +408,17 @@ class Playlist(GObject.GObject):
         self._source.query(
             query, self.METADATA_KEYS, options, _add_to_playlist_cb, None)
 
-    def rename(self, new_name):
+    @GObject.Property(type=str, default=None)
+    def title(self):
+        """Playlist title
+
+        :returns: playlist title
+        :rtype: str
+        """
+        return self._title
+
+    @title.setter
+    def title(self, new_name):
         """Rename a playlist
 
         :param str new_name: new playlist name
@@ -426,11 +426,17 @@ class Playlist(GObject.GObject):
         self._window.notifications_popup.push_loading()
 
         def update_cb(conn, res, data):
-            # FIXME: Check for failure.
-            conn.update_finish(res)
-            # FIXME: Requery instead?
-            self.props.title = new_name
-            self._window.notifications_popup.pop_loading()
+            try:
+                conn.update_finish(res)
+            except GLib.Error as e:
+                self._log.warning(
+                    "Unable to rename playlist from {} to {}: {}".format(
+                        self._title, new_name, e.message))
+            else:
+                self._title = new_name
+            finally:
+                self._window.notifications_popup.pop_loading()
+                self.thaw_notify()
 
         query = """
         INSERT OR REPLACE {
@@ -451,6 +457,7 @@ class Playlist(GObject.GObject):
             'playlist_id': self.props.pl_id
         }
 
+        self.freeze_notify()
         self._tracker.update_async(
             query, GLib.PRIORITY_LOW, None, update_cb, None)
 
@@ -687,9 +694,6 @@ class Playlist(GObject.GObject):
 class SmartPlaylist(Playlist):
     """Base class for smart playlists"""
 
-    def __repr__(self):
-        return "<SmartPlaylist>"
-
     def __init__(self, **args):
         super().__init__(**args)
 
@@ -704,7 +708,7 @@ class SmartPlaylist(Playlist):
 
             def _add_to_model(source, op_id, media, remaining, error):
                 if error:
-                    print("ERROR", error)
+                    self._log.warning("Error: {}".format(error))
                     self._window.notifications_popup.pop_loading()
                     self.emit("playlist-loaded")
                     return
@@ -725,52 +729,6 @@ class SmartPlaylist(Playlist):
 
         return self._model
 
-    def update(self):
-        """Updates playlist model."""
-        if self._model is None:
-            return
-
-        new_model_medias = []
-
-        def _fill_new_model(source, op_id, media, remaining, error):
-            if error:
-                return
-
-            if not media:
-                self._finish_update(new_model_medias)
-                return
-
-            new_model_medias.append(media)
-
-        options = self._fast_options.copy()
-        self._source.query(
-            self.props.query, self.METADATA_KEYS, options, _fill_new_model)
-
-    def _finish_update(self, new_model_medias):
-        if not new_model_medias:
-            self._model.remove_all()
-            self.props.count = 0
-            return
-
-        current_models_ids = [coresong.props.media.get_id()
-                              for coresong in self._model]
-        new_model_ids = [media.get_id() for media in new_model_medias]
-
-        idx_to_delete = []
-        for idx, media_id in enumerate(current_models_ids):
-            if media_id not in new_model_ids:
-                idx_to_delete.insert(0, idx)
-
-        for idx in idx_to_delete:
-            self._model.remove(idx)
-            self.props.count -= 1
-
-        for idx, media in enumerate(new_model_medias):
-            if media.get_id() not in current_models_ids:
-                coresong = CoreSong(media, self._coreselection, self._grilo)
-                self._model.append(coresong)
-                self.props.count += 1
-
 
 class MostPlayed(SmartPlaylist):
     """Most Played smart playlist"""
@@ -780,7 +738,7 @@ class MostPlayed(SmartPlaylist):
 
         self.props.tag_text = "MOST_PLAYED"
         # TRANSLATORS: this is a playlist name
-        self.props.title = _("Most Played")
+        self._title = _("Most Played")
         self.props.query = """
         SELECT
             rdf:type(?song)
@@ -805,7 +763,7 @@ class MostPlayed(SmartPlaylist):
         }
         ORDER BY DESC(?count) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            "location_filter": TrackerWrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter()
         }
 
 
@@ -817,7 +775,7 @@ class NeverPlayed(SmartPlaylist):
 
         self.props.tag_text = "NEVER_PLAYED"
         # TRANSLATORS: this is a playlist name
-        self.props.title = _("Never Played")
+        self._title = _("Never Played")
         self.props.query = """
         SELECT
             rdf:type(?song)
@@ -841,7 +799,7 @@ class NeverPlayed(SmartPlaylist):
             %(location_filter)s
         } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            "location_filter": TrackerWrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter()
         }
 
 
@@ -853,7 +811,7 @@ class RecentlyPlayed(SmartPlaylist):
 
         self.props.tag_text = "RECENTLY_PLAYED"
         # TRANSLATORS: this is a playlist name
-        self.props.title = _("Recently Played")
+        self._title = _("Recently Played")
 
         sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
         days_difference = 7
@@ -887,7 +845,7 @@ class RecentlyPlayed(SmartPlaylist):
         } ORDER BY DESC(?last_played) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'compare_date': compare_date,
-            "location_filter": TrackerWrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter()
         }
 
 
@@ -899,7 +857,7 @@ class RecentlyAdded(SmartPlaylist):
 
         self.props.tag_text = "RECENTLY_ADDED"
         # TRANSLATORS: this is a playlist name
-        self.props.title = _("Recently Added")
+        self._title = _("Recently Added")
 
         sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
         days_difference = 7
@@ -932,7 +890,7 @@ class RecentlyAdded(SmartPlaylist):
         } ORDER BY DESC(tracker:added(?song)) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'compare_date': compare_date,
-            "location_filter": TrackerWrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter()
         }
 
 
@@ -944,7 +902,7 @@ class Favorites(SmartPlaylist):
 
         self.props.tag_text = "FAVORITES"
         # TRANSLATORS: this is a playlist name
-        self.props.title = _("Favorite Songs")
+        self._title = _("Favorite Songs")
         self.props.query = """
             SELECT
                 rdf:type(?song)
@@ -970,5 +928,5 @@ class Favorites(SmartPlaylist):
                 %(location_filter)s
             } ORDER BY DESC(tracker:added(?song))
         """.replace('\n', ' ').strip() % {
-            "location_filter": TrackerWrapper.location_filter()
+            "location_filter": self._tracker_wrapper.location_filter()
         }
