@@ -25,7 +25,7 @@
 import math
 
 from gettext import gettext as _
-from gi.repository import GLib, GObject, Gtk
+from gi.repository import Gdk, GLib, GObject, Gtk
 
 from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.albumcover import AlbumCover
@@ -42,16 +42,13 @@ class AlbumsView(Gtk.Stack):
     __gtype_name__ = "AlbumsView"
 
     search_mode_active = GObject.Property(type=bool, default=False)
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
     _scrolled_window = Gtk.Template.Child()
     _flowbox = Gtk.Template.Child()
+    _flowbox_long_press = Gtk.Template.Child()
 
-    def __repr__(self):
-        return '<AlbumsView>'
-
-    def __init__(self, application, player=None):
+    def __init__(self, application):
         """Initialize AlbumsView
 
         :param application: The Application object
@@ -62,23 +59,28 @@ class AlbumsView(Gtk.Stack):
         self.name = "albums"
         self.title = _("Albums")
 
+        self._application = application
         self._window = application.props.window
         self._headerbar = self._window._headerbar
         self._adjustment_timeout_id = None
         self._viewport = self._scrolled_window.get_child()
         self._widget_counter = 1
+        self._ctrl_hold = False
 
-        model = self._window._app.props.coremodel.props.albums_sort
+        model = self._application.props.coremodel.props.albums_sort
         self._flowbox.bind_model(model, self._create_widget)
+        self._flowbox.set_hadjustment(self._scrolled_window.get_hadjustment())
+        self._flowbox.set_vadjustment(self._scrolled_window.get_vadjustment())
         self._flowbox.connect("child-activated", self._on_child_activated)
 
-        self.connect("notify::selection-mode", self._on_selection_mode_changed)
-
         self.bind_property(
-            'selection-mode', self._window, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL)
+            "selection-mode", self._window, "selection-mode",
+            GObject.BindingFlags.DEFAULT)
 
-        self._album_widget = AlbumWidget(application.props.player, self)
+        self._window.connect(
+            "notify::selection-mode", self._on_selection_mode_changed)
+
+        self._album_widget = AlbumWidget(self._application)
         self._album_widget.bind_property(
             "selection-mode", self, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -147,8 +149,14 @@ class AlbumsView(Gtk.Stack):
         return GLib.SOURCE_REMOVE
 
     def _on_selection_mode_changed(self, widget, data=None):
+        selection_mode = self._window.props.selection_mode
+        if (selection_mode == self.props.selection_mode
+                or self.get_parent().get_visible_child() != self):
+            return
+
+        self.props.selection_mode = selection_mode
         if not self.props.selection_mode:
-            self.unselect_all()
+            self.deselect_all()
 
     def _on_search_mode_changed(self, klass, param):
         if (not self.props.search_mode_active
@@ -166,7 +174,7 @@ class AlbumsView(Gtk.Stack):
 
         # NOTE: Adding SYNC_CREATE here will trigger all the nested
         # models to be created. This will slow down initial start,
-        # but will improve initial 'selecte all' speed.
+        # but will improve initial 'select all' speed.
         album_widget.bind_property(
             "selected", corealbum, "selected",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -198,18 +206,56 @@ class AlbumsView(Gtk.Stack):
         self._headerbar.props.title = corealbum.props.title
         self._headerbar.props.subtitle = corealbum.props.artist
 
+    @Gtk.Template.Callback()
+    def _on_flowbox_press_begin(self, gesture, sequence):
+        event = gesture.get_last_event(sequence)
+        ok, state = event.get_state()
+        if ((ok is True
+             and state == Gdk.ModifierType.CONTROL_MASK)
+                or self.props.selection_mode is True):
+            self._flowbox.props.selection_mode = Gtk.SelectionMode.MULTIPLE
+            if state == Gdk.ModifierType.CONTROL_MASK:
+                self._ctrl_hold = True
+
+    @Gtk.Template.Callback()
+    def _on_flowbox_press_cancel(self, gesture, sequence):
+        self._flowbox.props.selection_mode = Gtk.SelectionMode.NONE
+
+    @Gtk.Template.Callback()
+    def _on_selected_children_changed(self, flowbox):
+        if self._flowbox.props.selection_mode == Gtk.SelectionMode.NONE:
+            return
+
+        if self.props.selection_mode is False:
+            self.props.selection_mode = True
+
+        with self._application.props.coreselection.freeze_notify():
+            if self._ctrl_hold is False:
+                self.deselect_all()
+            for child in self._flowbox.get_selected_children():
+                if self._ctrl_hold is True:
+                    child.props.selected = not child.props.selected
+                else:
+                    child.props.selected = True
+
+        self._ctrl_hold = False
+        self._flowbox.props.selection_mode = Gtk.SelectionMode.NONE
+
     def _toggle_all_selection(self, selected):
+        """Selects or deselects all items.
         """
-        Selects or unselects all items without sending the notify::active
-        signal for performance purposes.
-        """
-        with self._window._app.props.coreselection.freeze_notify():
-            for child in self._flowbox.get_children():
-                child.props.selected = selected
-                child.props.corealbum.props.selected = selected
+        with self._application.props.coreselection.freeze_notify():
+            if self.get_visible_child() == self._album_widget:
+                if selected is True:
+                    self._album_widget.select_all()
+                else:
+                    self._album_widget.deselect_all()
+            else:
+                for child in self._flowbox.get_children():
+                    child.props.selected = selected
 
     def select_all(self):
         self._toggle_all_selection(True)
 
-    def unselect_all(self):
+    def deselect_all(self):
         self._toggle_all_selection(False)
