@@ -22,16 +22,11 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import logging
-
 from gi.repository import GObject, Gtk
 
-from gnomemusic import log
 from gnomemusic.player import PlayerPlaylist
 from gnomemusic.widgets.artistalbumwidget import ArtistAlbumWidget
 
-logger = logging.getLogger(__name__)
-
 
 class ArtistAlbumsWidget(Gtk.ListBox):
     """Widget containing all albums by an artist
@@ -43,13 +38,8 @@ class ArtistAlbumsWidget(Gtk.ListBox):
 
     __gtype_name__ = 'ArtistAlbumsWidget'
 
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
-    def __repr__(self):
-        return '<ArtistAlbumsWidget>'
-
-    @log
     def __init__(
             self, coreartist, application, selection_mode_allowed=False):
         super().__init__()
@@ -67,8 +57,6 @@ class ArtistAlbumsWidget(Gtk.ListBox):
         self._songs_grid_size_group = Gtk.SizeGroup.new(
             Gtk.SizeGroupMode.HORIZONTAL)
 
-        self._model.props.model.connect_after(
-            "items-changed", self. _on_model_items_changed)
         self.bind_model(self._model, self._add_album)
 
         self.get_style_context().add_class("artist-albums-widget")
@@ -82,7 +70,7 @@ class ArtistAlbumsWidget(Gtk.ListBox):
 
         coremodel = self._application.props.coremodel
 
-        def _on_playlist_loaded(klass):
+        def _on_playlist_loaded(klass, playlist_type):
             self._player.play(song_widget.props.coresong)
             coremodel.disconnect(signal_id)
 
@@ -90,6 +78,9 @@ class ArtistAlbumsWidget(Gtk.ListBox):
         coremodel.set_player_model(PlayerPlaylist.Type.ARTIST, self._model)
 
     def _add_album(self, corealbum):
+        row = Gtk.ListBoxRow()
+        row.props.selectable = False
+
         widget = ArtistAlbumWidget(
             corealbum, self._selection_mode_allowed,
             self._songs_grid_size_group, self._cover_size_group)
@@ -99,27 +90,21 @@ class ArtistAlbumsWidget(Gtk.ListBox):
             GObject.BindingFlags.BIDIRECTIONAL
             | GObject.BindingFlags.SYNC_CREATE)
 
+        row.add(widget)
         self._widgets.append(widget)
         widget.connect("song-activated", self._song_activated)
 
-        return widget
+        return row
 
-    def _on_model_items_changed(self, model, position, removed, added):
-        for i in range(model.get_n_items()):
-            row = self.get_row_at_index(i)
-            row.props.selectable = False
-
-    @log
     def select_all(self):
         """Select all items"""
         for widget in self._widgets:
             widget.select_all()
 
-    @log
-    def select_none(self):
+    def deselect_all(self):
         """Deselect all items"""
         for widget in self._widgets:
-            widget.select_none()
+            widget.deselect_all()
 
     @GObject.Property(type=str, flags=GObject.ParamFlags.READABLE)
     def artist(self):
